@@ -14,12 +14,11 @@
 #  - "rc"    : further release candidates on RC branch (e.g. from "nm-1-26" branch
 #              tag "1.26-rc2" with version number 1.25.91).
 #  - "major" : on stable branch do a major release (e.g. on "nm-1-26" branch
-#              release "1.26.0", followed by "1.26.1-dev").
+#              release "1.26.0").
 #              You should do a "major-post" release right a "major" release.
 #  - "major-post": after a "major" release, merge the release branch with main and
 #              do another devel snapshot on main (e.g. do "1.27.1-dev" release).
-#  - "minor" : on a stable branch do a minor release (e.g. "1.26.4" on "nm-1-26"
-#              branch and bump to "1.26.5-dev").
+#  - "minor" : on a stable branch do a minor release (e.g. "1.26.4" on "nm-1-26").
 #
 # Requisites:
 #
@@ -28,8 +27,7 @@
 #   * Run in a "clean" environment, i.e. no unusual environment variables set, on a recent
 #     Fedora, with suitable dependencies installed.
 #
-#   * First, ensure that you have a valid Gitlab's private token for gitlab.freedestkop.org
-#     stored in ~/.config/nm-release-token, or pass one with --gitlab-token argument.
+#   * First, ensure that you have ssh keys for "master.gnome.org" installed (and ssh-agent running).
 #     Also, ensure you have a GPG key that you want to use for signing. Also, have gpg-agent running
 #     and possibly configure `git config --get user.signingkey` for the proper key.
 #
@@ -41,13 +39,9 @@
 #
 # Run with --no-test to do the actual release.
 
-fail_msg() {
+die() {
     echo -n "FAIL: "
     echo_color 31 "$@"
-}
-
-die() {
-    fail_msg "$@"
     exit 1
 }
 
@@ -69,13 +63,12 @@ print_usage() {
     echo "     [--no-check-gitlab] \\"
     echo "     [--no-check-news] \\"
     echo "     [--no-warn-publish-docs] \\"
-    echo "     [--gitlab-token <private_gitlab_token>] \\"
 }
 
 die_help() {
     print_usage
     echo
-    sed -e '2,/^$/!d' -e 's/^#\($\| \)/  /' "$BASH_SOURCE"
+    sed -e '/^# /,/# Run with --no-test/!d' -e 's/^#\($\| \)/  /' "$BASH_SOURCE"
     exit 0
 }
 
@@ -131,7 +124,7 @@ check_gitlab_pipeline() {
     local SHA="$2"
     local PIPELINE_ID
 
-    PIPELINE_ID="$(curl --no-progress-meter "https://gitlab.freedesktop.org/api/v4/projects/411/pipelines?ref=$BRANCH&sha=$SHA&source=push&order_by=id" 2>/dev/null | jq '.[0].id')"
+    PIPELINE_ID="$(curl --no-progress-meter "https://gitlab.freedesktop.org/api/v4/projects/411/pipelines?ref=$BRANCH&sha=$SHA&order_by=id" 2>/dev/null | jq '.[0].id')"
     if ! [[ $PIPELINE_ID =~ [0-9]+ ]] ; then
         echo "Cannot find pipeline for branch $BRANCH. Check \"https://gitlab.freedesktop.org/NetworkManager/NetworkManager/pipelines?page=1&scope=branches&ref=$BRANCH\""
         return 1
@@ -252,11 +245,6 @@ while [ "$#" -ge 1 ]; do
         --help|-h)
             die_help
             ;;
-        --gitlab-token)
-            [ "$#" -ge 1 ] || die_usage "provide a value for --gitlab-token"
-            GITLAB_TOKEN="$1"
-            shift
-            ;;
         devel|rc1|rc|major|major-post|minor)
             [ -z "$RELEASE_MODE" ] || die_usage "duplicate release-mode"
             RELEASE_MODE="$A"
@@ -307,8 +295,7 @@ RC_VERSION=
 RELEASE_BRANCH=
 case "$RELEASE_MODE" in
     minor)
-        number_is_even "${VERSION_ARR[1]}" &&
-        number_is_odd  "${VERSION_ARR[2]}" || die "cannot do minor release on top of version $VERSION_STR"
+        number_is_even "${VERSION_ARR[1]}" || die "cannot do minor release on top of version $VERSION_STR"
         [ "$CUR_BRANCH" != main ] || die "cannot do a minor release on main"
         ;;
     devel)
@@ -379,8 +366,7 @@ if [ -n "$RELEASE_BRANCH" ]; then
 fi
 
 if [ "$ALLOW_LOCAL_BRANCHES" != 1 ]; then
-    # The release process changed a bit in nm-1-50, so previous versions has to be compared to nm-1-48, not to main
-    cmp <(git show "$ORIGIN/nm-1-48:contrib/fedora/rpm/release.sh") "$BASH_SOURCE_ABSOLUTE" || die "$BASH_SOURCE is not identical to \`git show \"$ORIGIN/nm-1-48:contrib/fedora/rpm/release.sh\"\`"
+    cmp <(git show "$ORIGIN/main:contrib/fedora/rpm/release.sh") "$BASH_SOURCE_ABSOLUTE" || die "$BASH_SOURCE is not identical to \`git show \"$ORIGIN/main:contrib/fedora/rpm/release.sh\"\`"
 fi
 
 if ! check_news "$RELEASE_MODE" "@{VERSION_ARR[@]}" ; then
@@ -425,7 +411,7 @@ fi
 
 if [ $CHECK_GITLAB = 1 ]; then
     if ! check_gitlab_pipeline "$CUR_BRANCH" "$CUR_HEAD" ; then
-        echo "Check the pipelines for branch \"$CUR_BRANCH\" at https://gitlab.freedesktop.org/NetworkManager/NetworkManager/-/pipelines?ref=$CUR_BRANCH&source=push"
+        echo "Check the pipelines for branch \"$CUR_BRANCH\" at https://gitlab.freedesktop.org/NetworkManager/NetworkManager/-/pipelines?ref=$CUR_BRANCH"
         echo "Wait for pipeline with \`ci-fairy wait-for-pipeline --project NetworkManager/NetworkManager --sha \"$CUR_HEAD\"\`"
         die "It seems not all gitlab-ci jobs were running/succeeding. Skip this check with --no-check-gitlab"
     fi
@@ -443,19 +429,13 @@ case "$RELEASE_MODE" in
     minor)
         set_version_number "${VERSION_ARR[0]}" "${VERSION_ARR[1]}" $(("${VERSION_ARR[2]}" + 1))
         git commit -m "release: bump version to ${VERSION_ARR[0]}.${VERSION_ARR[1]}.$(("${VERSION_ARR[2]}" + 1))" -a || die "failed to commit release"
-        set_version_number "${VERSION_ARR[0]}" "${VERSION_ARR[1]}" $(("${VERSION_ARR[2]}" + 2))
-        git commit -m "release: bump version to ${VERSION_ARR[0]}.${VERSION_ARR[1]}.$(("${VERSION_ARR[2]}" + 2)) (development)" -a || die "failed to commit devel version bump"
 
         b="${VERSION_ARR[0]}.${VERSION_ARR[1]}.$(("${VERSION_ARR[2]}" + 1))"
-        git tag -s -a -m "Tag $b" "$b" HEAD~ || die "failed to tag release"
+        git tag -s -a -m "Tag $b" "$b" HEAD || die "failed to tag release"
         BRANCHES+=("$b")
         CLEANUP_REFS+=("refs/tags/$b")
         BUILD_TAG="$b"
-        b="${VERSION_ARR[0]}.${VERSION_ARR[1]}.$(("${VERSION_ARR[2]}" + 2))"
-        git tag -s -a -m "Tag $b (development)" "$b-dev" HEAD || die "failed to tag devel version"
-        BRANCHES+=("$b-dev")
-        CLEANUP_REFS+=("refs/tags/$b-dev")
-        TAR_VERSION="$BUILD_TAG"
+        TAR_VERSION="$b"
         ;;
     devel)
         set_version_number "${VERSION_ARR[0]}" "${VERSION_ARR[1]}" $(("${VERSION_ARR[2]}" + 1))
@@ -494,20 +474,12 @@ case "$RELEASE_MODE" in
         ;;
     major)
         b="${VERSION_ARR[0]}.$((${VERSION_ARR[1]} + 1)).0"
-        b2="${VERSION_ARR[0]}.$((${VERSION_ARR[1]} + 1)).1"
-
         set_version_number "${VERSION_ARR[0]}" "$((${VERSION_ARR[1]} + 1))" 0
         git commit -m "release: bump version to $b" -a || die "failed to commit major version bump"
+
         git tag -s -a -m "Tag $b" "$b" HEAD || die "failed to tag release"
         BRANCHES+=("$b")
         CLEANUP_REFS+=("refs/tags/$b")
-
-        set_version_number "${VERSION_ARR[0]}" "$((${VERSION_ARR[1]} + 1))" 1
-        git commit -m "release: bump version to $b2 (development)" -a || die "failed to commit another bump after major version bump"
-        git tag -s -a -m "Tag $b (development)" "$b2-dev" HEAD || die "failed to tag release"
-        BRANCHES+=("$b2-dev")
-        CLEANUP_REFS+=("refs/tags/$b2-dev")
-
         BUILD_TAG="$b"
         TAR_VERSION="$b"
         ;;
@@ -539,23 +511,27 @@ case "$RELEASE_MODE" in
 esac
 
 build_tag() {
-    local BUILD_TAG="$1"
-    local TAR_FILE="NetworkManager-$2.tar.xz"
-    local SUM_FILE="$TAR_FILE.sha256sum"
-
     git checkout "$BUILD_TAG" || die "failed to checkout $BUILD_TAG"
+
     ./contrib/fedora/rpm/build_clean.sh -r || die "build release failed"
-    cp "./build/meson-dist/$TAR_FILE" /tmp/ || die "failed to copy $TAR_FILE to /tmp"
-    cp "./build/meson-dist/$SUM_FILE" /tmp/ || die "failed to copy $SUM_FILE to /tmp"
+
+    test -f "./build/meson-dist/$RELEASE_FILE" \
+    || die "release file \"./build/meson-dist/$RELEASE_FILE\" not found"
+
+    cp "./build/meson-dist/$RELEASE_FILE" /tmp/ || die "failed to copy release tarball to /tmp"
+
+    if test -f "./build/meson-dist/$RELEASE_FILE.sig" ; then
+        cp "./build/meson-dist/$RELEASE_FILE.sig" /tmp/ || die "failed to copy signature for tarball to /tmp"
+    fi
+
     git clean -fdx
 }
 
-RELEASE_TAR_VERSIONS=()
-RELEASE_TAGS=()
+RELEASE_FILES=()
 if [ -n "$BUILD_TAG" ]; then
-    build_tag "$BUILD_TAG" "$TAR_VERSION"
-    RELEASE_TAR_VERSIONS+=("$TAR_VERSION")
-    RELEASE_TAGS+=("$BUILD_TAG")
+    RELEASE_FILE="NetworkManager-$TAR_VERSION.tar.xz"
+    RELEASE_FILES+=("$RELEASE_FILE")
+    build_tag
 fi
 git checkout -B "$CUR_BRANCH" "$TMP_BRANCH" || die "cannot checkout $CUR_BRANCH"
 
@@ -565,6 +541,9 @@ if [ "$RELEASE_MODE" = rc1 ]; then
     git branch "$RELEASE_BRANCH" "$TMP_BRANCH" || die "cannot checkout $CUR_BRANCH"
     BRANCHES+=( "$RELEASE_BRANCH" )
     CLEANUP_REFS+=( "refs/heads/$RELEASE_BRANCH" )
+fi
+
+if [ "$RELEASE_MODE" = rc1 ]; then
     git checkout "$TMP_BRANCH"
     b="${VERSION_ARR[0]}.$((${VERSION_ARR[1]} + 2)).0"
     set_version_number "${VERSION_ARR[0]}" "$((${VERSION_ARR[1]} + 2))" 0
@@ -574,86 +553,29 @@ if [ "$RELEASE_MODE" = rc1 ]; then
     CLEANUP_REFS+=("refs/tags/$b-dev")
     BUILD_TAG="$b-dev"
     TAR_VERSION="$b"
-    build_tag "$BUILD_TAG" "$TAR_VERSION"
-    RELEASE_TAR_VERSIONS+=("$TAR_VERSION")
-    RELEASE_TAGS+=("$BUILD_TAG")
+    RELEASE_FILE="NetworkManager-$TAR_VERSION.tar.xz"
+    RELEASE_FILES+=("$RELEASE_FILE")
+    build_tag
     git checkout -B "$CUR_BRANCH" "$TMP_BRANCH" || die "cannot checkout $CUR_BRANCH"
 fi
 
-if [[ $GITLAB_TOKEN == "" ]]; then
-    [[ -r ~/.config/nm-release-token ]] || die "cannot read ~/.config/nm-release-token"
-    GITLAB_TOKEN=$(< ~/.config/nm-release-token)
+if ! [ "$DRY_RUN" = 0 ]; then
+    ssh master.gnome.org true || die "failed to \`ssh master.gnome.org\`"
 fi
 
-# This step is not necessary for authentication, we use it only to provide a meaningful error message.
-curl --request GET --header "PRIVATE-TOKEN: $GITLAB_TOKEN" \
-    "https://gitlab.freedesktop.org/api/v4/personal_access_tokens/self" &>/dev/null \
-    || die "failed to authenticate at gitlab.freedesktop.org with the private token"
+for r in "${RELEASE_FILES[@]}"; do
+    do_command rsync -va --append-verify -P "/tmp/$r" master.gnome.org: || die "failed to rsync \"/tmp/$r\""
+done
 
 do_command git push "$ORIGIN" "${BRANCHES[@]}" || die "failed to to push branches ${BRANCHES[@]} to $ORIGIN"
 
-CREATE_RELEASE_FAIL=0
-for I in "${!RELEASE_TAR_VERSIONS[@]}"; do
-    TAR_FILE="NetworkManager-${RELEASE_TAR_VERSIONS[$I]}.tar.xz"
-    SUM_FILE="$TAR_FILE.sha256sum"
-    BUILD_TAG="${RELEASE_TAGS["$I"]}"
-    FAIL=0
-
-    # upload tarball and checksum file as generic packages
-    for F in "$TAR_FILE" "$SUM_FILE"; do
-        do_command curl --location --fail-with-body --header "PRIVATE-TOKEN: $GITLAB_TOKEN" \
-            --upload-file "/tmp/$F" \
-            "https://gitlab.freedesktop.org/api/v4/projects/411/packages/generic/NetworkManager/$BUILD_TAG/$F" \
-            || FAIL=1
-
-        if [[ $FAIL = 1 ]]; then
-            fail_msg "failed to upload $F"
-            CREATE_RELEASE_FAIL=1
-            break
-        fi
-    done
-
-    [[ $FAIL = 1 ]] && continue
-
-    # create release
-    do_command curl --location  --header 'Content-Type: application/json' --header "PRIVATE-TOKEN: $GITLAB_TOKEN" \
-         --request POST "https://gitlab.freedesktop.org/api/v4/projects/411/releases" \
-         --data "$(cat <<END
-            {
-                "name": "NetworkManager $BUILD_TAG",
-                "tag_name": "$BUILD_TAG", 
-                "assets": {
-                    "links": [
-                        {
-                            "name": "NetworkManager $BUILD_TAG tarball with docs",
-                            "url": "https://gitlab.freedesktop.org/api/v4/projects/411/packages/generic/NetworkManager/$BUILD_TAG/$TAR_FILE",
-                            "direct_asset_path": "/$TAR_FILE",
-                            "link_type":"package"
-                        },
-                        {
-                            "name": "NetworkManager $BUILD_TAG tarball sha256sum",
-                            "url": "https://gitlab.freedesktop.org/api/v4/projects/411/packages/generic/NetworkManager/$BUILD_TAG/$SUM_FILE",
-                            "direct_asset_path": "/$SUM_FILE",
-                            "link_type":"package"
-                        },
-                        {
-                            "name": "NEWS",
-                            "url": "https://gitlab.freedesktop.org/NetworkManager/NetworkManager/-/blob/$BUILD_TAG/NEWS?ref_type=tags",
-                            "direct_asset_path": "/NEWS",
-                            "link_type":"other"
-                        }
-                    ]
-                }
-            }
-END
-            )" || FAIL=1
-
-    if [[ $? != 0 ]]; then
-        fail_msg "failed to create NetworkManager $BUILD_TAG release"
-        CREATE_RELEASE_FAIL=1
-        continue
-    fi
+FAIL=0
+for r in "${RELEASE_FILES[@]}"; do
+    do_command ssh master.gnome.org ftpadmin install --unattended "$r" || FAIL=1
 done
+if [ "$FAIL" = 1 ]; then
+    die "ftpadmin install failed. This was the last step. Invoke the command manually"
+fi
 
 CLEANUP_CHECKOUT_BRANCH=
 if [ "$DRY_RUN" = 0 ]; then
@@ -664,7 +586,3 @@ else
     git checkout -B "$CUR_BRANCH" "$CUR_HEAD" || die "cannot reset $CUR_BRANCH to $CUR_HEAD"
     echo "delete reference. Restore with $(echo_color 36 -n git checkout -B "\"$CUR_BRANCH\"" "$H")"
 fi
-
-if [[ $CREATE_RELEASE_FAIL == 1 ]]; then
-    die "failed creating the release at gitlab.freedesktop.org. This was the last step, create it manually from the web UI"
-fi
\ No newline at end of file
