@@ -135,12 +135,12 @@ else
         NMTST_USE_VALGRIND=0
     fi
 
-    if [ -z "${NMTST_LIBTOOL+x}" ]; then
-        NMTST_LIBTOOL=(sh "$SCRIPT_PATH/../libtool" "--mode=execute")
-    elif [ -z "$NMTST_LIBTOOL" ]; then
-        NMTST_LIBTOOL=()
-    else
+    if [ -n "$NMTST_LIBTOOL" ] ; then
         NMTST_LIBTOOL=("$NMTST_LIBTOOL" "--mode=execute")
+    elif [ -z "${NMTST_LIBTOOL+x}" -a -f "$SCRIPT_PATH/../libtool" ]; then
+        NMTST_LIBTOOL=(sh "$SCRIPT_PATH/../libtool" "--mode=execute")
+    else
+        NMTST_LIBTOOL=()
     fi
     TEST_ARGV=()
     unset TEST
@@ -302,8 +302,8 @@ if ! _is_true "$NMTST_USE_VALGRIND" 0; then
     export NM_TEST_UNDER_VALGRIND=0
     "${NMTST_DBUS_RUN_SESSION[@]}" "$TEST" "${TEST_ARGV[@]}"
     r=$?
-    [ $r == 0 ] || die "exec \"$TEST\" failed with exit code $r"
-    exit 0
+    [ $r == 0 -o $r == 77 ] || die "exec \"$TEST\" failed with exit code $r"
+    exit $r
 fi
 
 if [[ -z "${NMTST_VALGRIND}" ]]; then
